"use client";

import { useState, useCallback } from "react";
import { collection, query, where, getDocs, writeBatch, doc } from "firebase/firestore";
import { db } from "@/lib/firebase";
import { Attendance, WeeklyAttendance } from "@/lib/types";

export const useAttendance = () => {
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);

  const markMultipleAttendances = useCallback(async (attendances: Omit<Attendance, "id">[]) => {
    setLoading(true);
    setError(null);
    try {
      const batch = writeBatch(db);
      
      // Get all student IDs we're processing
      const studentIds = attendances.map(a => a.studentId);
      const date = attendances[0]?.date;
      const classId = attendances[0]?.classId;

      // Query for existing attendance records for these students on this date
      const existingRecordsQuery = query(
        collection(db, "attendance"),
        where("classId", "==", classId),
        where("date", "==", date),
        where("studentId", "in", studentIds)
      );
      
      const querySnapshot = await getDocs(existingRecordsQuery);
      const existingRecords = querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Attendance[];
      
      // Create a map of studentId to existing document
      const existingRecordsMap = new Map<string, Attendance>();
      existingRecords.forEach(record => {
        existingRecordsMap.set(record.studentId, record);
      });

      // Process each attendance record
      attendances.forEach(attendance => {
        const existingRecord = existingRecordsMap.get(attendance.studentId);
        
        if (existingRecord) {
          // Update existing document
          const docRef = doc(db, "attendance", existingRecord.id);
          batch.update(docRef, {
            status: attendance.status,
            remarks: attendance.remarks,
            markedBy: attendance.markedBy,
            updatedAt: new Date().toISOString()
          });
        } else {
          // Create new document
          const docRef = doc(collection(db, "attendance"));
          batch.set(docRef, {
            ...attendance,
            createdAt: new Date().toISOString()
          });
        }
      });
      
      await batch.commit();
    } catch (err) {
      console.error("Error marking attendance:", err);
      setError("Failed to mark attendance");
      throw err;
    } finally {
      setLoading(false);
    }
  }, []);

  const getAttendanceByClass = useCallback(async (classId: string, startDate: string, endDate: string) => {
    setLoading(true);
    setError(null);
    try {
      const attendanceQuery = query(
        collection(db, "attendance"),
        where("classId", "==", classId),
        where("date", ">=", startDate),
        where("date", "<=", endDate)
      );
      
      const querySnapshot = await getDocs(attendanceQuery);
      return querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Attendance[];
    } catch (err) {
      console.error("Error getting attendance:", err);
      setError("Failed to get attendance");
      throw err;
    } finally {
      setLoading(false);
    }
  }, []);

  const getAttendanceByStudent = useCallback(async (studentId: string, startDate: string, endDate: string) => {
    setLoading(true);
    setError(null);
    try {
      const attendanceQuery = query(
        collection(db, "attendance"),
        where("studentId", "==", studentId),
        where("date", ">=", startDate),
        where("date", "<=", endDate)
      );
      
      const querySnapshot = await getDocs(attendanceQuery);
      return querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Attendance[];
    } catch (err) {
      console.error("Error getting student attendance:", err);
      setError("Failed to get student attendance");
      throw err;
    } finally {
      setLoading(false);
    }
  }, []);

  const getWeeklyAttendance = useCallback(async (classId: string, startDate: string, endDate: string) => {
    setLoading(true);
    setError(null);
    try {
      const attendanceQuery = query(
        collection(db, "attendance"),
        where("classId", "==", classId),
        where("date", ">=", startDate),
        where("date", "<=", endDate)
      );
      
      const querySnapshot = await getDocs(attendanceQuery);
      const attendanceRecords = querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Attendance[];
      
      const weeklyData: Record<string, { present: number; absent: number; late: number }> = {};
      
      attendanceRecords.forEach(record => {
        const day = new Date(record.date).toLocaleDateString('en-US', { weekday: 'long' });
        
        if (!weeklyData[day]) {
          weeklyData[day] = { present: 0, absent: 0, late: 0 };
        }
        
        if (record.status === 'present') weeklyData[day].present += 1;
        else if (record.status === 'absent') weeklyData[day].absent += 1;
        else if (record.status === 'late') weeklyData[day].late += 1;
      });
      
      return Object.entries(weeklyData).map(([day, counts]) => ({
        day,
        ...counts
      })) as WeeklyAttendance[];
    } catch (err) {
      console.error("Error getting weekly attendance:", err);
      setError("Failed to get weekly attendance");
      throw err;
    } finally {
      setLoading(false);
    }
  }, []);

  const getAttendanceRate = useCallback(async (classId: string, startDate: string, endDate: string) => {
    try {
      const weeklyData = await getWeeklyAttendance(classId, startDate, endDate);
      let totalPresent = 0;
      let totalCount = 0;
      
      weeklyData.forEach(day => {
        totalPresent += day.present;
        totalCount += day.present + day.absent + day.late;
      });
      
      return totalCount > 0 ? (totalPresent / totalCount) * 100 : 0;
    } catch (err) {
      console.error("Error calculating attendance rate:", err);
      throw err;
    }
  }, [getWeeklyAttendance]);

  return {
    loading,
    error,
    markMultipleAttendances,
    getAttendanceByClass,
    getAttendanceByStudent,
    getWeeklyAttendance,
    getAttendanceRate
  };
};